CLI Code Helper 是一个基于 OpenAI API 的命令行工具，能够通过自然语言交互帮助用户解决编程问题、生成shell命令并执行。

# 功能特点
- 支持与 GPT 模型进行自然语言对话
- 可以生成并执行 shell 命令
- 支持流式输出模式
- 提供对话历史记录管理
- 支持多种 shell 环境（Bash、Zsh、Fish、PowerShell、Cmd）
- 提供丰富的交互命令
- 支持切换不同的 GPT 模型

# 系统架构

项目采用模块化设计，主要包含以下模块：

## 1. API 模块 (`api/`)
- `client.rs`: OpenAI API 客户端实现
- `config.rs`: 配置管理
- `models.rs`: 数据模型定义

## 2. CLI 模块 (`cli/`)
- `interactive.rs`: 交互模式实现
- `shell.rs`: Shell 命令执行

## 3. 工具模块 (`utils/`)
- `display.rs`: 显示管理器

# 配置说明

配置文件存储在用户目录下的 `.config/cli-chat/config.json`，包含：
- API 密钥（可通过环境变量 `OPENAI_API_KEY` 设置）
- 使用的模型配置

## 支持的模型

- `gpt-3.5-turbo`: 最快速且最具成本效益
- `gpt-4o`: 功能更强大但较慢
- `gpt-4o-mini`: 最新模型，具有改进的功能

## 命令列表

| 命令 | 说明 |
|------|------|
| `/new` | 开始新对话 |
| `/stream` | 切换流式输出模式 |
| `/clear` | 清屏 |
| `/context` | 显示对话历史 |
| `/help` | 显示帮助信息 |
| `/exit` | 退出程序(输入q也可) |
| `/model` | 显示可用模型 |
| `!` | 生成并执行 shell 命令（例如：`! how to list files`）|

# 特色功能

## Shell 命令生成与执行
- 使用 `!` 前缀可以触发命令生成
- 自动检测当前 shell 环境
- 执行前会请求用户确认
- 支持显示命令执行结果

## 对话管理
- 保持对话上下文
- 支持查看历史记录
- 可以随时开始新对话
- 显示对话轮次计数

## 显示效果
- 彩色输出界面
- 打字机效果的消息显示
- 思考状态显示动画
- 清晰的对话历史格式化

# 使用示例

1. 开始普通对话：
```bash
> 你好
Assistant: 你好！有什么我可以帮你的吗？
```

2. 生成并执行命令：
```bash
>!如何查看当前目录下的文件
Generated command:
ls -la

Do you want to execute this command? [y/N]:
```

3. 切换模型：
```bash
> /model gpt-4o
Model changed to: gpt-4o
```

# 部署说明

1. 确保已设置 OpenAI API 密钥
2. 运行程序：
```bash
cargo run
```

或者使用

```bash
cargo build --release && ./target/release/cli-helper 
```

下面是一个示例

编译环境为Vmware下的Debian 12.8 , x86-64 

## 指令生成

# ![截屏2024-12-13 22.58.13](/Users/sagechen/Library/Application Support/typora-user-images/截屏2024-12-13 22.58.13.png)

![截屏2024-12-13 22.58.46](/Users/sagechen/Library/Application Support/typora-user-images/截屏2024-12-13 22.58.46.png)

## 问题回答

![截屏2024-12-13 23.00.55](/Users/sagechen/Library/Application Support/typora-user-images/截屏2024-12-13 23.00.55.png)

## 历史对话

![截屏2024-12-13 23.02.19](/Users/sagechen/Library/Application Support/typora-user-images/截屏2024-12-13 23.02.19.png)

## 切换模型

![截屏2024-12-13 23.03.39](/Users/sagechen/Library/Application Support/typora-user-images/截屏2024-12-13 23.03.39.png)

# 环境要求

- Rust 开发环境
- OpenAI API 密钥
- 支持的 shell 环境之一

# 注意事项

- 请确保 API 密钥安全存储
- 建议在执行生成的 shell 命令前仔细检查
- 流式输出模式可能会增加 API 调用成本


# 项目日志

- [2024-11-20] 项目启动，完成了基本的对话功能,完成了流式传输、多轮对话的功能
- [2024-11-22] 增加了命令生成并执行的功能

- [2024-11-25] 修复了一些bug
