# 智能命令行助手 (Command Line AI Assistant)

基于百度文心一言API的智能命令行助手，提供自然语言命令理解和执行支持。

## 项目目标

- 开发基于百度文心一言API的智能命令行助手
- 提供自然语言命令理解和执行支持
- 实现智能命令补全和上下文理解
- 提供深度的命令执行结果分析

## 技术栈

- **编程语言**：Python 3.8+
- **API**：百度文心一言 (Yi-34B-Chat模型)
- **主要依赖库**：
  1. `requests>=2.31.0`：API调用
  2. `rich>=13.7.0`：终端界面美化
  3. `click>=8.1.7`：命令行界面
  4. `python-dotenv>=1.0.0`：环境变量管理
  5. `colorama>=0.4.6`：终端颜色支持
  6. `prompt-toolkit>=3.0.41`：命令补全
  7. `dataclasses-json>=0.6.3`：数据序列化

## 项目结构

```
hw1/
├── src/                    # 源代码目录
│   ├── main.py            # 主程序入口
│   ├── api_client.py      # API调用封装
│   ├── utils.py           # 工具函数
│   ├── command_completer.py # 命令补全模块
│   ├── command_analyzer.py  # 命令分析模块
│   └── context_manager.py   # 上下文管理模块
├── tests/                  # 测试目录
│   ├── test_api_client.py   # API客户端测试
│   ├── test_command_parsing.py # 命令解析测试
│   ├── test_error_handling.py  # 错误处理测试
│   ├── test_execution_flow.py  # 执行流程测试
│   ├── test_integration.py     # API集成测试
│   ├── test_security_checks.py # 安全检查测试
│   └── test_utils.py          # 工具函数测试
├── config/                 # 配置文件目录
│   ├── .env               # 环境变量配置
│   └── settings.json      # 其他配置
├── .command_history.json  # 命令历史记录
├── .context.json         # 上下文信息
└── requirements.txt      # 项目依赖
```

## 功能特性

### 1. 核心功能
- API集成
  - 使用 Yi-34B-Chat 模型
  - 获取和管理 access token
  - 错误处理和响应解析

- 命令行交互
  - 自然语言理解
  - 命令解释和执行
  - 系统命令执行
  - 命令历史记录

### 2. 智能补全
- 系统命令自动补全
- 文件路径智能补全
- 命令用法提示
- 实时补全建议

### 3. 上下文理解
- 对话历史管理
- 命令执行状态跟踪
- 相关性分析
- 智能建议生成

### 4. 命令分析
- 执行结果深度分析
- 错误原因诊断
- 性能指标统计
- 优化建议生成

### 5. 安全特性
- 命令安全检查
- API错误处理
- 异常捕获
- 权限控制

### 6. 用户体验
- 终端界面美化
- 进度条显示
- 彩色输出
- 友好的交互提示

## 测试覆盖

项目包含以下测试模块，确保各个组件的正确性：

### 1. 命令解析测试 (test_command_parsing.py)
- 测试命令分析功能
  - 验证成功命令的分析结果
  - 验证命令类型识别
  - 测试错误命令的分析

### 2. 错误处理测试 (test_error_handling.py)
- 测试命令执行功能
  - 验证有效命令的执行
  - 验证无效命令的错误处理

### 3. 执行流程测试 (test_execution_flow.py)
- 测试命令助手的完整流程
  - 验证命令执行
  - 测试命令历史记录功能

### 4. API集成测试 (test_integration.py)
- 测试与文心一言API的集成
  - 验证聊天完成功能
  - 模拟API响应处理

### 5. 安全检查测试 (test_security_checks.py)
- 测试命令安全性检查
  - 验证安全命令的识别
  - 验证危险命令的识别
  - 测试特殊字符和重定向的处理

## 运行测试

在项目根目录下运行以下命令来执行所有测试：

```bash
python -m unittest discover tests
```

## 配置说明

### API配置
在 `config/.env` 文件中配置以下内容：
```env
API_KEY=yYLPWex4ioszMAhtmCZCMOdZ
SECRET_KEY=gNjLwZdgIi0vobu34h7aNS1WDdEF4q26
MAX_HISTORY_SIZE=100
COMMAND_HISTORY_FILE=.command_history.json
CONTEXT_FILE=.context.json
```

## 快速开始

1. 安装依赖：
```bash
pip install -r requirements.txt
```

2. 运行测试：
```bash
python -m unittest discover tests
```

3. 启动程序：
```bash
python src/main.py
```

## 使用限制

1. API调用频率限制
2. 命令执行仅支持基本系统命令
3. 部分高级功能可能需要管理员权限

## 更新日志

### v1.1.0 (2024-12-07)
- 添加智能命令补全功能
- 实现上下文管理系统
- 增加命令执行结果分析
- 完善测试用例
- 优化用户界面

### v1.0.0 (2024-12-06)
- 初始版本发布
- 基础功能实现
- API集成完成
