import unittest
import sys
from io import StringIO
from datetime import datetime
import colorama
from colorama import Fore, Style

# 初始化colorama
colorama.init()

def run_tests_with_output():
    # 创建测试加载器
    loader = unittest.TestLoader()
    
    # 从tests目录发现所有测试
    suite = loader.discover('tests')
    
    # 创建StringIO对象来捕获输出
    output = StringIO()
    runner = unittest.TextTestRunner(
        stream=output,
        verbosity=2,
        descriptions=True
    )
    
    # 运行测试
    result = runner.run(suite)
    
    # 获取捕获的输出
    output_str = output.getvalue()
    
    # 生成带时间戳的文件名
    timestamp = datetime.now().strftime('%Y%m%d_%H%M%S')
    filename = f'test_results_{timestamp}.txt'
    
    # 保存原始输出
    with open(filename, 'w', encoding='utf-8') as f:
        f.write(output_str)
    
    # 在控制台显示彩色输出
    for line in output_str.splitlines():
        if 'ok' in line.lower():
            print(Fore.GREEN + line + Style.RESET_ALL)
        elif 'error' in line.lower() or 'fail' in line.lower():
            print(Fore.RED + line + Style.RESET_ALL)
        else:
            print(line)
    
    return result

if __name__ == '__main__':
    result = run_tests_with_output()
    sys.exit(not result.wasSuccessful())
