"""
命令补全模块
"""
import os
import re
from typing import List, Optional
from prompt_toolkit import PromptSession
from prompt_toolkit.completion import Completer, Completion

class CommandCompleter(Completer):
    def __init__(self):
        self.common_commands = {
            'windows': [
                'dir', 'cd', 'copy', 'del', 'mkdir', 'rmdir', 'type',
                'ping', 'ipconfig', 'tasklist', 'taskkill', 'systeminfo'
            ],
            'linux': [
                'ls', 'cd', 'cp', 'rm', 'mkdir', 'rmdir', 'cat',
                'ping', 'ifconfig', 'ps', 'kill', 'uname'
            ]
        }
        
        self.command_descriptions = {
            'dir': '列出目录内容',
            'cd': '改变当前目录',
            'copy': '复制文件',
            'del': '删除文件',
            'mkdir': '创建新目录',
            'rmdir': '删除目录',
            'type': '显示文件内容',
            'ping': '测试网络连接',
            'ipconfig': '显示网络配置',
            'tasklist': '显示进程列表',
            'taskkill': '终止进程',
            'systeminfo': '显示系统信息'
        }

    def get_completions(self, document, complete_event):
        word = document.get_word_before_cursor()
        
        # 补全命令
        for command in self.common_commands['windows']:
            if command.startswith(word):
                display = f"{command:<15} - {self.command_descriptions.get(command, '')}"
                yield Completion(
                    command,
                    start_position=-len(word),
                    display=display,
                    display_meta=self.command_descriptions.get(command, '')
                )
        
        # 补全路径
        if word.startswith(('.', '/', '\\')):
            current_dir = os.getcwd()
            path = word.replace('/', os.sep).replace('\\', os.sep)
            
            if os.path.isabs(path):
                base_path = os.path.dirname(path)
            else:
                base_path = os.path.join(current_dir, os.path.dirname(path))
            
            try:
                for item in os.listdir(base_path):
                    full_path = os.path.join(base_path, item)
                    if item.startswith(os.path.basename(word)):
                        display = f"{item}{'/' if os.path.isdir(full_path) else ''}"
                        yield Completion(
                            display,
                            start_position=-len(os.path.basename(word))
                        )
            except OSError:
                pass

def create_session() -> PromptSession:
    """创建带有命令补全功能的会话"""
    return PromptSession(
        completer=CommandCompleter(),
        complete_while_typing=True
    )
