"""
API客户端测试
"""
import unittest
from unittest.mock import patch, MagicMock
import sys
import os

# 添加项目根目录到 Python 路径
sys.path.append(os.path.dirname(os.path.dirname(os.path.abspath(__file__))))

from src.api_client import APIClient

class TestAPIClient(unittest.TestCase):
    def setUp(self):
        self.api_client = APIClient()

    @patch('requests.post')
    def test_get_access_token(self, mock_post):
        # 模拟成功响应
        mock_response = MagicMock()
        mock_response.json.return_value = {"access_token": "test_token"}
        mock_post.return_value = mock_response

        token = self.api_client.get_access_token()
        self.assertEqual(token, "test_token")

    @patch('requests.post')
    def test_chat_completion(self, mock_post):
        # 模拟成功响应
        mock_response = MagicMock()
        mock_response.json.return_value = {"result": "test response"}
        mock_post.return_value = mock_response

        messages = [{"role": "user", "content": "test message"}]
        response = self.api_client.chat_completion(messages)
        self.assertEqual(response["result"], "test response")

if __name__ == '__main__':
    unittest.main()
