import unittest
import os
from src.main import CommandAssistant

class TestExecutionFlow(unittest.TestCase):
    def setUp(self):
        self.assistant = CommandAssistant()
        # 确保测试文件存在
        self.test_file = "test_output.txt"
        if not os.path.exists(self.test_file):
            with open(self.test_file, "w") as f:
                f.write("test content")

    def tearDown(self):
        # 清理测试文件
        if os.path.exists(self.test_file):
            os.remove(self.test_file)
        
        # 清理命令助手实例
        if hasattr(self, 'assistant'):
            # 保存并关闭历史记录
            if hasattr(self.assistant, 'history'):
                self.assistant.history.save_to_file('.command_history.json')
            
            # 保存并关闭上下文
            if hasattr(self.assistant, 'context'):
                self.assistant.context.save_context('.context.json')

    def test_command_execution(self):
        # 测试基本命令执行
        result = self.assistant.execute_command_safely(f"type {self.test_file}")
        
        # 测试命令历史记录
        last_commands = self.assistant.history.get_last()
        self.assertIn(f"type {self.test_file}", last_commands)

    def test_system_command_execution(self):
        # 测试系统命令执行
        result = self.assistant.execute_system_command(f"type {self.test_file}")
        self.assertTrue(result)

if __name__ == '__main__':
    unittest.main()
