"""
工具函数测试
"""
import unittest
import sys
import os
import tempfile
import json
import time

# 添加项目根目录到 Python 路径
sys.path.append(os.path.dirname(os.path.dirname(os.path.abspath(__file__))))

from src.utils import get_system_info, execute_command, is_safe_command, CommandHistory

class TestUtils(unittest.TestCase):
    def test_get_system_info(self):
        system_info = get_system_info()
        self.assertIn(system_info, ['Windows', 'Linux', 'Darwin'])

    def test_execute_command(self):
        # 测试有效命令
        code, stdout, stderr = execute_command('echo test')
        self.assertEqual(code, 0)
        self.assertIn('test', stdout)

        # 测试无效命令
        code, stdout, stderr = execute_command('invalid_command')
        self.assertNotEqual(code, 0)

    def test_is_safe_command(self):
        # 测试安全命令
        self.assertTrue(is_safe_command('dir'))
        self.assertTrue(is_safe_command('echo hello'))
        
        # 测试危险命令
        self.assertFalse(is_safe_command('rm -rf /'))
        self.assertFalse(is_safe_command('format c:'))

class TestCommandHistory(unittest.TestCase):
    def setUp(self):
        self.history = CommandHistory(max_size=3)

    def test_add_and_get_last(self):
        self.history.add('cmd1')
        self.history.add('cmd2')
        self.assertEqual(self.history.get_last(), ['cmd2'])
        self.assertEqual(self.history.get_last(2), ['cmd1', 'cmd2'])

    def test_max_size(self):
        self.history.add('cmd1')
        self.history.add('cmd2')
        self.history.add('cmd3')
        self.history.add('cmd4')  # 应该移除 cmd1
        self.assertEqual(len(self.history.history), 3)
        self.assertEqual(self.history.get_last(3), ['cmd2', 'cmd3', 'cmd4'])

    def test_save_and_load(self):
        temp_dir = tempfile.mkdtemp()
        temp_file = os.path.join(temp_dir, 'test_history.json')
        
        try:
            self.history.add('cmd1')
            self.history.add('cmd2')
            
            # 保存到临时文件
            self.history.save_to_file(temp_file)
            
            # 创建新的历史记录并加载
            new_history = CommandHistory(max_size=3)
            new_history.load_from_file(temp_file)
            
            self.assertEqual(new_history.history, self.history.history)
            
        finally:
            # 确保文件句柄已关闭后再清理
            if os.path.exists(temp_file):
                try:
                    os.remove(temp_file)
                except:
                    pass
            try:
                os.rmdir(temp_dir)
            except:
                pass

if __name__ == '__main__':
    unittest.main()
